# Author:      Chris Wailes <chris.wailes@gmail.com>
# Project:     Filigree
# Date:        2013/4/19
# Description: This file specifies the version number of Filigree.

module Filigree
	VERSION = '0.3.3'
end
