# Author:      Chris Wailes <chris.wailes@gmail.com>
# Project:     Ruby Language Toolkit
# Date:        2012/03/08
# Description: This file specifies the version number of RLTK.

module RLTK
	# The version number of the RLTK library.
	VERSION             = '3.0.1'
	# The version of LLVM targeted by RLTK.
	LLVM_TARGET_VERSION = '3.4'
end
