$LOAD_PATH.unshift(File.expand_path("../ffi-1.9.17//Users/georgebrock/.rbenv/versions/2.4.2/lib/ruby/gems/2.4.0/extensions/x86_64-darwin-16/2.4.0-static/ffi-1.9.17", __FILE__))
$LOAD_PATH.unshift(File.expand_path("../ffi-1.9.17/lib", __FILE__))
$LOAD_PATH.unshift(File.expand_path("../filigree-0.3.3/lib", __FILE__))
$LOAD_PATH.unshift(File.expand_path("../rltk-3.0.1/lib", __FILE__))
