# Author:      Chris Wailes <chris.wailes@gmail.com>
# Project:     Ruby Language Toolkit
# Date:        2012/05/04
# Description: This file contains test suit for the RLTK LLVM bindigns.  It
#              requires the individual tests from their respective files.

############
# Requires #
############

# Ruby Language Toolkit
require 'cg/tc_llvm'
require 'cg/tc_module'

require 'cg/tc_basic_block'
require 'cg/tc_control_flow'
require 'cg/tc_function'
require 'cg/tc_generic_value'
require 'cg/tc_instruction'
require 'cg/tc_math'
require 'cg/tc_transforms'
require 'cg/tc_type'
require 'cg/tc_value'
